//
// Text World
//
// A sample program that's gradually evolving into
// a real text editor application.
//
// Written by: Eric Shepherd
//
/*
	Copyright 1999, Be Incorporated.   All Rights Reserved.
	This file may be used under the terms of the Be Sample Code License.
*/

#define DEBUG 0
#include <Debug.h>

#include <Application.h>
#include <Messenger.h>
#include <Message.h>
#include <Roster.h>
#include <Window.h>
#include <View.h>
#include <MenuBar.h>
#include <Menu.h>
#include <MenuItem.h>
#include <FilePanel.h>
#include <Path.h>
#include <Entry.h>
#include <TextView.h>
#include <ScrollView.h>
#include <String.h>
#include <string.h>
#include <stdio.h>

#include "constants.h"
#include "textapp.h"
#include "textwindow.h"

BRect windowRect(50,50,599,399);

//
// TextApp::TextApp
//
// The constructor for the TextApp class.  This
// will create our window.
//
TextApp::TextApp()
			: BApplication(APP_SIGNATURE) 
{
	
	RegisterDocumentFiletype();													// KIRILLA
	
	window_count = 0;			// No windows yet
	next_untitled_number = 1;	// Next window is "Untitled 1"
	
	// Create the Open file panel
	
	openPanel = new BFilePanel;
	
	new TextWindow(windowRect);
}


//
// TextApp::MessageReceived
//
// Handle incoming messages.  In particular, handle the
// WINDOW_REGISTRY_ADD and WINDOW_REGISTRY_SUB messages.
//
void TextApp::MessageReceived(BMessage *message) 
{
	switch(message->what) 
	{
		case WINDOW_REGISTRY_ADD:
			{
				bool need_id = false;
				BMessage reply(WINDOW_REGISTRY_ADDED);
				
				if (message->FindBool("need_id", &need_id) == B_OK) 
				{
					if (need_id) 
					{
						reply.AddInt32("new_window_number", next_untitled_number);
						next_untitled_number++;
					}
					window_count++;
				}
				reply.AddRect("rect", windowRect);
				windowRect.OffsetBy(20,20);
				message->SendReply(&reply);
				break;
			}
		case WINDOW_REGISTRY_SUB:
			window_count--;
			if (!window_count) 
			{
				Quit();
			}
			break;
		case MENU_FILE_OPEN:
			openPanel->Show();		// Show the file panel
			break;
		default:
			BApplication::MessageReceived(message);
			break;
	}
}

//
// TextApp::RefsReceived
//
// Handle a refs received message.
//
void TextApp::RefsReceived(BMessage *message) 
{
	entry_ref 	ref;		// The entry_ref to open
	status_t 	err;		// The error code
	int32		ref_num;	// The index into the ref list
	
	// Loop through the ref list and open each one

	ref_num = 0;
	do 
	{
		if ((err = message->FindRef("refs", ref_num, &ref)) != B_OK) 
		{
			return;
		}
		new TextWindow(windowRect, &ref);
		ref_num++;

	} while (1);
}


void TextApp::RegisterDocumentFiletype	(void)		// KIRILLA
{
	status_t	status	=	B_OK;
	char * buffer	=	new char [B_MIME_TYPE_LENGTH];
	buffer[B_MIME_TYPE_LENGTH - 1]	=	'\0';

	BMimeType	mime;
	mime.SetTo(DOCUMENT_FILETYPE);
	
	// install mimetype
	if (! mime.IsInstalled())				
	{
		mime.Install();
		
		// For some reason this code does not appear to run, ever.
		// Does the BeOS auto-install the apps supported type(s)
		// and set the preferred app on them?

		PRINT(("mime.Install();\n"));
	}
	
	// filename extension
	BMessage	ext_msg;

	status = mime.GetFileExtensions(& ext_msg);
	
	type_code	type	=	0;
	int32		count	=	0;
	ext_msg.GetInfo("extensions", & type, & count);	 // how many extensions does the filetype specify
	
	if ((status != B_OK) || count < 2) 							// it's our filetype, so we're not being
	{															// a bad citizen by making sure that 
																// these two filename extensions
		ext_msg.MakeEmpty();									// are coupled with the filetype.
		ext_msg.AddString("extensions", "twd");					// Unless of course, these extensions are 
		ext_msg.AddString("extensions", "TWD");					// used by some other, prominent filetype
		mime.SetFileExtensions(& ext_msg);						// 
																// if we were really anal about this
		PRINT(("mime.SetFileExtensions(& ext_msg);\n"));		// we'd check the exact extensions
	}															// and not just them being 2
	
	// short description
	status = mime.GetShortDescription(buffer);
	BString string	(buffer);
	
	if ((status != B_OK) || (string != "TextWorld Document"))	// it's our document type, so we're not being
	{															// a bad citizen by making sure that 
																// it's always called exactly that
		mime.SetShortDescription("TextWorld Document");
		
		PRINT(("mime.SetShortDescription(TextWorld Document);\n"));
	}
	
	// preferred application
	status = mime.GetPreferredApp(buffer);
	if (status != B_OK)										// the most correct thing would be to check	for 
	{														// B_ENTRY_NOT_FOUND, but we catch that and any
		mime.SetPreferredApp(APP_SIGNATURE);				// other failure by checking for (status != B_OK)
															// This is no life-support machine so we can afford
		PRINT(("mime.SetPreferredApp(APP_SIGNATURE);\n"));	// to crash & burn once every blue moon
	
		// For some reason this code does not appear
		// to be run the first time the app runs 
		// Does the BeOS auto-install the apps supported type(s)
		// and set the preferred app on them?
	}

	delete [] buffer;
}

//
// main
//
// The main() function's only real job in a basic BeOS
// application is to create the BApplication object
// and run it.
//
int main(void) {
	TextApp theApp;		// The application object
	theApp.Run();
	return 0;
}



